@@ -7,10 +7,9 @@
  * By Daniel Vérité, 2018-2023. See LICENSE.md
  */
 
-#include "postgres.h"
+#include "icu_ext.h"
 
 #include "catalog/pg_collation.h"
-#include "fmgr.h"
 #include "funcapi.h"
 #include "lib/stringinfo.h"
 #include "miscadmin.h"
@@ -30,8 +29,6 @@
 #include "unicode/utext.h"
 #include "unicode/uvernum.h"
 
-#include "icu_ext.h"
-
 PG_MODULE_MAGIC;
 
 PG_FUNCTION_INFO_V1(icu_version);
@@ -46,6 +43,9 @@ PG_FUNCTION_INFO_V1(icu_case_compare);
 PG_FUNCTION_INFO_V1(icu_sort_key);
 PG_FUNCTION_INFO_V1(icu_sort_key_coll);
 PG_FUNCTION_INFO_V1(icu_char_name);
+PG_FUNCTION_INFO_V1(icu_char_type);
+PG_FUNCTION_INFO_V1(icu_char_ublock_id);
+PG_FUNCTION_INFO_V1(icu_unicode_blocks);
 
 
 /*
@@ -58,6 +58,17 @@ char *icu_ext_timestamptz_format;
 UDateFormatStyle icu_ext_date_style = UDAT_DEFAULT;
 UDateFormatStyle icu_ext_timestamptz_style = UDAT_DEFAULT;
 
+static const char* general_category_types[] = {
+	"Cn",
+	"Lu", "Ll", "Lt", "Lm", "Lo",
+	"Mn", "Me", "Mc",
+	"Nd", "Nl", "No",
+	"Zs", "Zl", "Zp",
+	"Cc", "Cf", "Co", "Cs",
+	"Pd", "Ps", "Pe", "Pc", "Po",
+	"Sm", "Sc", "Sk", "So",
+	"Pi", "Pf"
+};
 
 void		_PG_init(void);
 
@@ -414,7 +425,6 @@ icu_collation_attributes(PG_FUNCTION_ARGS)
 		tuplestore_putvalues(tupstore, tupdesc, values, nulls);
 	}
 
-	tuplestore_donestoring(tupstore);
 	ucol_close(collator);
 
 	return (Datum) 0;
@@ -558,7 +568,6 @@ icu_locales_list(PG_FUNCTION_ARGS)
 		tuplestore_putvalues(tupstore, tupdesc, values, nulls);
 	}
 
-	tuplestore_donestoring(tupstore);
 	return (Datum) 0;
 }
 
@@ -756,7 +765,7 @@ Datum
 icu_sort_key(PG_FUNCTION_ARGS)
 {
 	text *txt = PG_GETARG_TEXT_PP(0);
-    UCollator *collator = ucollator_from_coll_id(PG_GET_COLLATION());
+	UCollator *collator = ucollator_from_coll_id(PG_GET_COLLATION());
 	int32_t o_len = 1024;		/* first attempt */
 	int32_t ulen;
 	UChar *ustring;
@@ -861,7 +870,7 @@ first_char32(BpChar* source)
 }
 
 /*
- * Return the Unicode name corresponding to the the input character.
+ * Return the Unicode name corresponding to the input character.
  */
 Datum
 icu_char_name(PG_FUNCTION_ARGS)
@@ -900,6 +909,104 @@ icu_char_name(PG_FUNCTION_ARGS)
 		PG_RETURN_TEXT_P(cstring_to_text(buffer));
 }
 
+/*
+ * Return the Unicode general category type of the input's code point
+ */
+Datum
+icu_char_type(PG_FUNCTION_ARGS)
+{
+	BpChar *source = PG_GETARG_BPCHAR_PP(0);
+	UChar32 first_char;
+	int8_t char_type;
+
+	first_char = first_char32(source);
+	char_type = u_charType(first_char);
+
+	if (char_type >= 0 && char_type < sizeof(general_category_types)/sizeof(general_category_types[0]))
+	{
+		PG_RETURN_TEXT_P(cstring_to_text(general_category_types[char_type]));
+	}
+	else
+	{
+		elog(ERROR, "unexpected return value of u_charType for codepoint: 0x%lx",
+			 (unsigned long)first_char);
+	}
+}
+
+/*
+ * Return the Unicode block number (ICU UBlockCode enum) of the
+ * input's code point.
+ */
+Datum
+icu_char_ublock_id(PG_FUNCTION_ARGS)
+{
+	BpChar *source = PG_GETARG_BPCHAR_PP(0);
+	UChar32 first_char;
+	UBlockCode block_id;
+
+	first_char = first_char32(source);
+	block_id = ublock_getCode(first_char);
+
+	PG_RETURN_INT16((int16)block_id);
+}
+
+/*
+ * Return the list of Unicode blocks with their internal ID in ICU
+ * (UBlockCode enum, matching the return values of
+ * icu_char_ublock_id()), and the block names.
+ */
+Datum
+icu_unicode_blocks(PG_FUNCTION_ARGS)
+{
+	ReturnSetInfo *rsinfo = (ReturnSetInfo *) fcinfo->resultinfo;
+	MemoryContext per_query_ctx;
+	MemoryContext oldcontext;
+	TupleDesc	tupdesc;
+	Tuplestorestate *tupstore;
+	Datum values[2];
+	bool nulls[2];
+
+	if (rsinfo == NULL || !IsA(rsinfo, ReturnSetInfo))
+		ereport(ERROR,
+				(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
+			 errmsg("set-valued function called in context that cannot accept a set")));
+
+	/* Switch into long-lived context to construct returned data structures */
+	per_query_ctx = rsinfo->econtext->ecxt_per_query_memory;
+	oldcontext = MemoryContextSwitchTo(per_query_ctx);
+
+	/* Build a tuple descriptor for our result type */
+	if (get_call_result_type(fcinfo, NULL, &tupdesc) != TYPEFUNC_COMPOSITE)
+		elog(ERROR, "return type must be a row type");
+
+	tupstore = tuplestore_begin_heap(true, false, work_mem);
+	rsinfo->returnMode = SFRM_Materialize;
+	rsinfo->setResult = tupstore;
+	rsinfo->setDesc = tupdesc;
+
+	MemoryContextSwitchTo(oldcontext);
+
+	memset(nulls, 0, sizeof(nulls));
+
+	for (int16 nblock = (int16)UBLOCK_NO_BLOCK ;
+		 nblock < (int16)UBLOCK_COUNT;
+		 nblock++)
+	{
+		const char* block_name =
+			u_getPropertyValueName(UCHAR_BLOCK,
+								   nblock,
+								   U_LONG_PROPERTY_NAME);
+		if (block_name != NULL)
+		{
+			values[0] = Int16GetDatum(nblock);
+			values[1] = CStringGetTextDatum(block_name);
+			tuplestore_putvalues(tupstore, tupdesc, values, nulls);
+		}
+		/* else nblock is out of range */
+	}
+
+	return (Datum) 0;
+}
 
 /*
  * Convert {full|medium|...} into an UDateFormatStyle value, or UDAT_NONE
