@@ -7,8 +7,10 @@
  * By Daniel Vérité, 2018-2023. See LICENSE.md
  */
 
+#include "icu_ext.h"
+
 /* Postgres includes */
-#include "postgres.h"
+
 #include "mb/pg_wchar.h"
 #include "utils/builtins.h"
 #include "utils/pg_locale.h"
@@ -19,7 +21,6 @@
 /* ICU includes */
 #include "unicode/unorm.h"
 
-#include "icu_ext.h"
 
 PG_FUNCTION_INFO_V1(icu_is_normalized);
 PG_FUNCTION_INFO_V1(icu_normalize);
@@ -137,7 +138,7 @@ icu_normalize(PG_FUNCTION_ARGS)
 Datum
 icu_is_normalized(PG_FUNCTION_ARGS)
 {
-    text *src_text = PG_GETARG_TEXT_PP(0);
+	text *src_text = PG_GETARG_TEXT_PP(0);
 	const char* arg_form = text_to_cstring(PG_GETARG_TEXT_PP(1));
 	norm_form_t form = name_to_norm(arg_form);
 	UErrorCode	status = U_ZERO_ERROR;
